import { Link } from "@components/Link";
import clsx from "clsx";

interface BaseProps {
  type: "primary" | "secondary" | "tertiary";
  size?: "xs" | "sm" | "md" | "lg" | "xl";
  className?: string;
  text?: string;
  icon?: React.ReactNode;
  iconRight?: boolean;
}

type ButtonOrLink =
  | {
      href?: never;
      onClick?: () => void;
      disabled?: boolean;
      buttonType?: "button" | "submit" | "reset";
      newTab?: never;
    }
  | {
      href: string;
      onClick?: never;
      disabled?: never;
      buttonType?: never;
      newTab?: boolean;
    };

type CircularOrRounded =
  | {
      circular?: true;
      rounded?: never;
      text?: never;
    }
  | {
      circular?: never;
      rounded?: true;
      text?: string;
    };

export type ButtonProps = BaseProps & ButtonOrLink & CircularOrRounded;

export function Button({
  text,
  type = "primary",
  size,
  rounded,
  circular,
  disabled,
  icon,
  iconRight,
  onClick,
  href,
  buttonType = "button",
  className = "",
  newTab,
}: ButtonProps) {
  const STYLE_MAPPING: Record<string, string> = {
    primary:
      "fs-bg-grey-900 hover:fs-bg-grey-700 focus:fs-bg-grey-700 fs-text-white hover:fs-text-grey-400 focus:fs-text-grey-400 fs-no-underline fs-not-prose",
    secondary:
      "fs-bg-grey-400 hover:fs-bg-grey-200 focus:fs-bg-grey-200 fs-text-grey-900 hover:fs-text-grey-500 focus:fs-text-grey-500 fs-no-underline fs-not-prose",
    tertiary:
      "fs-bg-white hover:fs-bg-grey-100 focus:fs-bg-grey-100 fs-text-grey-900 hover:fs-text-grey-500 focus:fs-text-grey-500 fs-no-underline fs-not-prose",
  };

  const SIZE_MAPPING: Record<string, string> = {
    xs: "fs-px-3 fs-py-2",
    sm: "fs-px-3.5 fs-py-2.5",
    md: "fs-px-4 fs-py-3",
    lg: "fs-px-5 fs-py-3.5",
    xl: "fs-px-6 fs-py-4",
    default:
      "fs-px-3 sm:fs-px-3.5 md:fs-px-4 lg:fs-px-5 xl:fs-px-6 fs-py-2 sm:fs-py-2.5 md:fs-py-3 lg:fs-py-3.5 xl:fs-py-4",
  };

  const TEXT_MAPPING: Record<string, string> = {
    xs: "fs-font-plex fs-font-medium fs-text-xs fs-leading-4",
    sm: "fs-font-plex fs-font-medium fs-text-sm fs-leading-4",
    md: "fs-font-plex fs-font-medium fs-text-sm fs-leading-5",
    lg: "fs-font-plex fs-font-medium fs-text-base fs-leading-6",
    xl: "fs-font-plex fs-font-medium fs-text-base fs-leading-6",
    default:
      "fs-font-plex fs-text-xs sm:fs-text-sm lg:fs-text-base fs-leading-4 md:fs-leading-5 lg:fs-leading-6 fs-font-medium",
  };

  const CIRCULAR_SIZE_MAPPING: Record<string, string> = {
    xs: "fs-size-7",
    sm: "fs-size-8",
    md: "fs-size-9",
    lg: "fs-size-10",
    xl: "fs-size-11",
    default: "fs-size-7 sm:fs-size-8 md:fs-size-9 lg:fs-size-10 xl:fs-size-11",
  };

  const CIRCULAR_STYLE_MAPPING: Record<string, string> = {
    primary:
      "fs-flex fs-items-center fs-justify-center fs-rounded-full fs-bg-grey-900 hover:fs-bg-grey-700 focus:fs-bg-grey-700 fs-text-white hover:fs-text-grey-400 focus:fs-text-grey-400",
    secondary:
      "fs-flex fs-items-center fs-justify-center fs-rounded-full fs-bg-grey-400 hover:fs-bg-grey-200 focus:fs-bg-grey-200 fs-text-grey-900 hover:fs-text-grey-500 focus:fs-text-grey-500",
    tertiary:
      "fs-flex fs-items-center fs-justify-center fs-rounded-full fs-bg-white hover:fs-bg-grey-100 focus:fs-bg-grey-100 fs-text-grey-900 hover:fs-text-grey-500 focus:fs-text-grey-500",
  };

  const ADDITIONAL_STYLE_MAPPING: Record<string, string> = {
    rounded: "fs-rounded-full",
    square: "fs-rounded",
    outline:
      "sm:focus:fs-ring-2 focus:fs-ring-offset-2 focus:fs-ring-offset-white focus:fs-ring-grey-900",
    transition: "fs-transition-all fs-ease-in-out fs-duration-200",
    disabled: "fs-opacity-50 fs-pointer-events-none",
  };

  const notCircularClasses = clsx(
    "fs-flex fs-items-center fs-justify-center fs-w-fit",
    ADDITIONAL_STYLE_MAPPING["outline"],
    rounded
      ? ADDITIONAL_STYLE_MAPPING["rounded"]
      : ADDITIONAL_STYLE_MAPPING["square"],
    ADDITIONAL_STYLE_MAPPING["transition"],
    size ? TEXT_MAPPING[size] : TEXT_MAPPING["default"],
    size ? SIZE_MAPPING[size] : SIZE_MAPPING["default"],
    STYLE_MAPPING[type],
  );

  const circularClasses = clsx(
    ADDITIONAL_STYLE_MAPPING["outline"],
    ADDITIONAL_STYLE_MAPPING["transition"],
    size ? CIRCULAR_SIZE_MAPPING[size] : CIRCULAR_SIZE_MAPPING["default"],
    CIRCULAR_STYLE_MAPPING[type],
  );

  const renderButtonContent = () => {
    return (
      <>
        {icon && (
          <span
            className={clsx(
              "fs-text-center",
              iconRight ? "fs-order-2 fs-ml-2.5" : "fs-order-1 fs-mr-2.5",
            )}
          >
            {icon}
          </span>
        )}
        <span
          className={clsx(
            "fs-text-center",
            iconRight ? "fs-order-1" : "fs-order-2",
          )}
        >
          {text}
        </span>
      </>
    );
  };

  return (
    <>
      {href ? (
        <Link
          to={href}
          className={clsx(
            circular ? circularClasses : notCircularClasses,
            className,
          )}
          {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
        >
          {circular ? icon : renderButtonContent()}
        </Link>
      ) : (
        <button
          className={clsx(
            circular ? circularClasses : notCircularClasses,
            disabled && ADDITIONAL_STYLE_MAPPING["disabled"],
            className,
          )}
          disabled={disabled}
          onClick={onClick}
          type={buttonType}
        >
          {circular ? icon : renderButtonContent()}
        </button>
      )}
    </>
  );
}
