import React from "react";
import { ISocialMediaItem, IPolicy, ISupportedBy } from "@interfaces/IFooter";
import { Link } from "@components/Link";
import { SocialMediaIcons } from "@components/SocialMediaIcons";
import { INestedMenuItem } from "@interfaces/IMenu";
import { ConditionalWrapper } from "./ConditionalWrapper";
import clsx from "clsx";

export interface FooterProps {
  menu: INestedMenuItem[];
  copyright?: string;
  policyLinks?: IPolicy[];
  socialMedia?: ISocialMediaItem[];
  socialMediaGrow?: boolean;
  supportedBy?: ISupportedBy[];
  newsletterSlot?: React.ReactNode;
}

export function Footer({
  menu,
  copyright,
  policyLinks,
  socialMedia,
  supportedBy,
  newsletterSlot,
  socialMediaGrow,
}: FooterProps) {
  return (
    <footer className="fs-bg-grey-900 fs-py-16">
      <div className="container">
        <Menu menu={menu} />
        <NewsletterAndSocialMedia
          socialMediaGrow={socialMediaGrow}
          socialMedia={socialMedia}
          newsletterSlot={newsletterSlot}
        />
        {supportedBy && (
          <SupportedBy title="Gefördert durch" items={supportedBy} />
        )}
        <BottomLine copyright={copyright} policyLinks={policyLinks} />
      </div>
    </footer>
  );
}

interface SupportedByProps {
  items?: ISupportedBy[];
  title: string;
}

function SupportedBy({ items, title }: SupportedByProps) {
  return (
    <div className="fs-mt-5">
      <h4 className="fs-font-normal fs-text-sm fs-text-grey-400">{title}</h4>
      {items && (
        <div className="fs-flex fs-flex-col sm:fs-flex-row fs-gap-5 fs-mt-5 fs-items-center">
          {items.map((item, index) => (
            <ConditionalWrapper
              key={index}
              condition={Boolean(item.href)}
              wrapper={(children) => (
                // eslint-disable-next-line react/jsx-no-target-blank
                <a target="_blank" rel="noopener" href={item.href}>
                  {children}
                </a>
              )}
            >
              <img src={item.img} alt={item.alt} className="fs-size-60" />
            </ConditionalWrapper>
          ))}
        </div>
      )}
    </div>
  );
}

interface MenuProps {
  menu: INestedMenuItem[];
}

function Menu({ menu }: MenuProps) {
  return (
    <div className="fs-w-full fs-flex fs-flex-col md:fs-flex-row md:fs-flex-wrap fs-font-plex">
      {menu?.map((item, index) => <SubMenu key={index} submenu={item} />)}
    </div>
  );
}

interface ISubMenu {
  submenu: INestedMenuItem;
}

function SubMenu({ submenu }: ISubMenu) {
  return (
    <div className="fs-w-full md:fs-w-1/3 lg:fs-w-1/5 md:fs-mt-0 fs-mb-16 md:fs-mb-8 lg:fs-mb-0">
      {submenu.children && submenu.children.length > 0 ? (
        <h5 className="fs-w-fit fs-text-sm fs-leading-5 fs-font-semibold fs-text-grey-50">
          {submenu.title}
        </h5>
      ) : (
        <Link
          to={submenu.target}
          className="fs-w-fit fs-text-sm fs-leading-5 fs-font-semibold fs-text-grey-50"
        >
          {submenu.title}
        </Link>
      )}
      {submenu.children && submenu.children.length > 0 && (
        <ul>
          {submenu.children.map((child, index) => (
            <li key={index} className="fs-w-fit fs-mt-5">
              <a
                href={child.target}
                className="fs-text-sm fs-leading-5 fs-font-normal fs-text-grey-400"
              >
                {child.title}
              </a>
            </li>
          ))}
        </ul>
      )}
    </div>
  );
}

interface NewsletterAndSocialMediaProps {
  socialMedia?: ISocialMediaItem[];
  newsletterSlot?: React.ReactNode;
  socialMediaGrow?: boolean;
}

function NewsletterAndSocialMedia({
  socialMedia,
  newsletterSlot,
  socialMediaGrow,
}: NewsletterAndSocialMediaProps) {
  return (
    <div className="fs-w-full fs-flex fs-flex-col md:fs-flex-row md:fs-justify-between fs-mt-0 md:fs-mt-12">
      <SocialMediaIcons
        grow={socialMediaGrow}
        icons={socialMedia}
        id="social-footer"
        className="fs-mr-4 fs-order-2 fs-flex-1 lg:fs-order-1 fs-mt-16 md:fs-mt-0 md:fs-max-w-sm"
        variant="light"
      />
      {newsletterSlot && (
        <div className="fs-w-full md:fs-w-3/5 lg:fs-w-1/2 xl:fs-w-[34%] 2xl:fs-w-[28%] fs-order-1 md:fs-order-2">
          {newsletterSlot}
        </div>
      )}
    </div>
  );
}

interface BottomLineProps {
  copyright?: string;
  policyLinks?: IPolicy[];
}

function BottomLine({ copyright, policyLinks }: BottomLineProps) {
  return (
    <div className="fs-w-full fs-flex fs-flex-col lg:fs-flex-row md:fs-justify-between fs-mt-16 md:fs-mt-10">
      {policyLinks?.length ? (
        <ul className="fs-font-plex fs-text-sm fs-leading-5 fs-font-normal fs-text-grey-400 fs-text-center lg:fs-text-left">
          {policyLinks?.map((policy, index) => (
            <li
              key={index}
              className={clsx(
                "fs-inline-block",
                index !== policyLinks?.length - 1 && "fs-border-r",
                index === 0 ? "fs-pr-4" : "fs-px-4",
              )}
            >
              <a href={policy.href}>{policy.title}</a>
            </li>
          ))}
        </ul>
      ) : (
        <div></div>
      )}
      {copyright && (
        <div className="fs-text-center lg:fs-text-left fs-mt-6 lg:fs-mt-0">
          <span className="fs-font-plex fs-text-sm fs-leading-5 fs-font-normal fs-text-grey-400">
            {copyright}
          </span>
        </div>
      )}
    </div>
  );
}
