import { Fragment } from "react";
import { Link } from "@components/Link";
import clsx from "clsx";

type PillType =
  | "success"
  | "danger"
  | "warning"
  | "info"
  | "dark"
  | "default"
  | "purple"
  | "pink"
  | "white";

type BaseProps = {
  title: string;
  rounded?: boolean;
  size: "sm" | "lg";
  className?: string;
  color?: string;
};

type TypeOrCustom =
  | {
      type?: never;
      customBgColor: string;
    }
  | {
      type: PillType;
      customBgColor?: never;
    };

type HrefOrOnClick =
  | {
      href?: string;
      onClick?: never;
      newTab?: boolean;
    }
  | {
      href?: never;
      onClick?: () => void;
      newTab?: never;
    };

type onIconClickOrNot =
  | {
      icon?: React.ReactNode;
      onIconClick?: () => void;
      iconRight?: boolean;
    }
  | {
      icon?: never;
      onIconClick?: never;
      iconRight?: never;
    };

export type PillProps = BaseProps &
  HrefOrOnClick &
  onIconClickOrNot &
  TypeOrCustom;

export function Pill({
  type = "default",
  href,
  title,
  icon,
  iconRight,
  rounded,
  size = "lg",
  newTab,
  onClick,
  onIconClick,
  className = "",
  customBgColor = "",
}: PillProps) {
  const STYLE_MAPPING: Record<string, string> = {
    success: "fs-bg-teal-100 fs-text-teal-700",
    danger: "fs-bg-red-100 fs-text-red-700",
    warning: "fs-bg-amber-100 fs-text-amber-700",
    info: "fs-bg-blue-50 fs-text-blue-700",
    dark: "fs-bg-grey-900 fs-text-grey-100",
    default: "fs-bg-grey-200 fs-text-grey-800",
    purple: "fs-bg-purple-100 fs-text-purple-800",
    pink: "fs-bg-pink-100 fs-text-pink-800",
    white: "fs-bg-white fs-text-grey-900",
    custom: "",
  };

  const ICON_STYLE_MAPPING: Record<string, string> = {
    success: "fs-text-teal-400",
    danger: "fs-text-red-400",
    warning: "fs-text-amber-400",
    info: "fs-text-blue-400",
    dark: "fs-text-grey-500",
    default: "fs-text-grey-500",
    purple: "fs-text-purple-400",
    pink: "fs-text-pink-400",
    white: "fs-text-grey-900",
    custom: "fs-text-grey-900",
  };

  const SIZE_MAPPING: Record<string, string> = {
    sm: "fs-w-fit fs-flex fs-items-center fs-px-2.5 sm:fs-px-3 fs-py-0.5 fs-text-xs fs-leading-4 fs-font-medium fs-font-plex fs-no-underline fs-not-prose",
    lg: "fs-w-fit fs-flex fs-items-center fs-px-2.5 fs-sm:px-3.5 md:fs-py-1 fs-text-sm fs-leading-5 fs-font-medium fs-font-plex fs-no-underline fs-not-prose",
  };

  const OTHER_STYLES_MAPPING: Record<string, string> = {
    rounded: "fs-rounded-full",
    square: "fs-rounded",
  };

  const iconPositionClasses =
    icon && iconRight
      ? "fs-pr-1.5 sm:fs-pr-1"
      : icon && !iconRight
        ? "fs-pl-1 sm:fs-pl-1.5"
        : "";

  const renderPillContent = () => {
    return (
      <Fragment>
        {icon && (
          <span
            onClick={onIconClick}
            className={clsx(
              ICON_STYLE_MAPPING[type],
              iconRight ? "fs-order-2 fs-ml-0.5" : "fs-order-1 fs-mr-0.5",
              onIconClick && "fs-cursor-pointer",
            )}
          >
            {icon}
          </span>
        )}
        <span className={clsx(iconRight ? "fs-order-1" : "fs-order-2")}>
          {title}
        </span>
      </Fragment>
    );
  };

  return href ? (
    <Link
      to={href}
      style={customBgColor ? { backgroundColor: customBgColor } : {}}
      className={clsx(
        STYLE_MAPPING[type],
        SIZE_MAPPING[size],
        rounded
          ? OTHER_STYLES_MAPPING["rounded"]
          : OTHER_STYLES_MAPPING["square"],
        iconPositionClasses,
      )}
      {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
    >
      {renderPillContent()}
    </Link>
  ) : (
    <div
      style={customBgColor ? { backgroundColor: customBgColor } : {}}
      onClick={() => onClick && !onIconClick && onClick()}
      className={clsx(
        STYLE_MAPPING[type],
        SIZE_MAPPING[size],
        rounded
          ? OTHER_STYLES_MAPPING["rounded"]
          : OTHER_STYLES_MAPPING["square"],
        iconPositionClasses,
        onClick && !onIconClick ? "fs-cursor-pointer" : "fs-cursor-default",
        className,
      )}
    >
      {renderPillContent()}
    </div>
  );
}
