import { useEffect, useState, useRef } from "react";
import { Pill } from "@components/Pill";
import { CommonUtils } from "@utils/CommonUtils";
import { Close } from "@carbon/icons-react";
import { ChevronRight } from "@carbon/icons-react";
import clsx from "clsx";

export interface PillCloudProps {
  activeTags?: string[];
  tags?: Array<{ target?: string; title: string }>;
  onTagEnable?: (tag: string) => void;
  onTagDisable?: (tag: string) => void;
}

export function PillCloud({
  activeTags = [],
  tags,
  onTagEnable = () => {},
  onTagDisable = () => {},
}: PillCloudProps) {
  const slidingFactor = 1.2; // Increase/decrease this value to make the tag cloud scroll with wider/narrower width
  const sliderRef = useRef<HTMLDivElement>(null);
  const [scrollX, setScrollX] = useState(0);
  const [isScrolledToRight, setIsScrolledToRight] = useState(false);
  const [isScrolledToLeft, setIsScrolledToLeft] = useState(true);
  const [shouldShowNavigationalButtons, setShouldShowNavigationalButtons] =
    useState(true);

  const handleTagClick = (tag: string) => {
    activeTags.includes(tag) ? onTagDisable(tag) : onTagEnable(tag);
  };

  const handleButtonScroll = (px: number) => {
    if (sliderRef.current) {
      sliderRef.current.scrollTo({
        top: 0,
        left: sliderRef.current.scrollLeft + px,
        behavior: "smooth",
      });
    }
  };

  const handlSwipeScroll = () => {
    if (sliderRef.current) {
      setIsScrolledToLeft(!(sliderRef.current.scrollLeft > 0));
      setIsScrolledToRight(
        sliderRef.current.scrollLeft + sliderRef.current.clientWidth >=
          sliderRef.current.scrollWidth,
      );
    }
  };

  const calculateScrollX = () => {
    if (sliderRef.current) {
      const widthPerSlide =
        (sliderRef.current.scrollWidth - sliderRef.current.clientWidth) *
        slidingFactor;
      setScrollX(widthPerSlide);
    }
  };

  useEffect(() => {
    calculateScrollX();
  }, []);

  useEffect(() => {
    calculateScrollX();
  }, [sliderRef]);

  useEffect(() => {
    if (sliderRef.current) {
      setShouldShowNavigationalButtons(
        sliderRef.current.scrollWidth > sliderRef.current.clientWidth,
      );
    }
  }, [tags]);

  return (
    <div className="fs-relative fs-w-full">
      {shouldShowNavigationalButtons && (
        <>
          <NavigationalButton
            className={
              isScrolledToLeft ? "fs-opacity-0 fs-invisible" : "fs-opacity-100"
            }
            direction="left"
            onClick={() => handleButtonScroll(-scrollX)}
          />
          <NavigationalButton
            className={
              isScrolledToRight ? "fs-opacity-0 fs-invisible" : "fs-opacity-100"
            }
            direction="right"
            onClick={() => handleButtonScroll(scrollX)}
          />
        </>
      )}
      <div
        ref={sliderRef}
        onScroll={handlSwipeScroll}
        className="fs-w-full fs-flex fs-gap-3 fs-overflow-y-hidden fs-overflow-x-auto fs-whitespace-nowrap fs-no-scrollbar"
      >
        {Boolean(tags?.length) &&
          tags?.map((tag) => {
            const isActiveTag = activeTags.includes(tag.title);
            const tagId = CommonUtils.generateID();
            return isActiveTag ? (
              <Pill
                key={tagId}
                type="dark"
                title={tag.title}
                onClick={() => handleTagClick(tag.title)}
                size="lg"
                icon={
                  <Close className="fs-size-3.5 sm:fs-size-4 fs-text-white fs-mx-0 sm:fs-mx-1" />
                }
                iconRight
                rounded
              />
            ) : (
              <Pill
                key={tagId}
                type="default"
                title={tag.title}
                onClick={() => handleTagClick(tag.title)}
                size="lg"
                rounded
              />
            );
          })}
      </div>
    </div>
  );
}

interface NavigationalButtonProps {
  direction: "left" | "right";
  onClick: () => void;
  className?: string;
}

function NavigationalButton({
  direction,
  onClick,
  className = "",
}: NavigationalButtonProps) {
  const STYLE_MAPPING: Record<string, string> = {
    left: "fs-hidden sm:fs-block fs-absolute fs-z-[1] fs-h-[101%] -fs-left-1 fs-flex fs-justify-center fs-items-center fs-outline-0 fs-px-3 fs-bg-white after:fs-content-[''] after:fs-absolute after:fs-z-[2] after:fs-left-full after:fs-top-0 after:fs-w-8 after:fs-h-full after:fs-bg-gradient-to-l after:fs-from-transparent after:fs-to-white",
    right:
      "fs-hidden sm:fs-block fs-absolute fs-z-[1] fs-h-[101%] -fs-right-1 fs-flex fs-justify-center fs-items-center fs-outline-0 fs-px-3 fs-bg-white before:fs-content-[''] before:fs-absolute before:fs-z-[2] before:fs-right-full before:fs-top-0 before:fs-w-8 before:fs-h-full before:fs-bg-gradient-to-r before:fs-from-transparent before:fs-to-white",
  };
  return (
    <div className={clsx(STYLE_MAPPING[direction], className)}>
      <button onClick={onClick} className="fs-h-full fs-bg-white fs-outline-0">
        <ChevronRight
          size={20}
          className={direction === "left" ? "-fs-scale-100" : "fs-scale-100"}
        />
      </button>
    </div>
  );
}
